"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const archive_1 = require("./archive");
const error_1 = require("./error");
const execute_1 = __importDefault(require("./execute"));
const exists_1 = __importDefault(require("./exists"));
const offline_1 = require("./offline");
const parse_1 = __importDefault(require("./parse"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
/**
 * Uses local git installation to clone a repository to the destination.
 */
async function clone(repository, options = {}) {
    const info = (0, parse_1.default)(repository, options);
    const path = (0, archive_1.getArchivePath)(info, options);
    let order = [];
    const local = async () => (0, exists_1.default)(path);
    const remote = async () => new Promise((resolve, reject) => {
        const child = (0, cross_spawn_1.default)('git', ['clone', info.href, path]);
        child.on('close', (code) => {
            if (code === 0) {
                resolve(path);
            }
            else {
                reject(new error_1.GitlyCloneError('Failed to clone the repository'));
            }
        });
    });
    if ((await (0, offline_1.isOffline)()) || options.cache) {
        order = [local];
    }
    else if (options.force || ['master', 'main'].includes(info.type)) {
        order = [remote, local];
    }
    try {
        const result = await (0, execute_1.default)(order);
        if (typeof result === 'boolean') {
            return path;
        }
        return result;
    }
    catch (error) {
        if (options.throw) {
            throw error;
        }
    }
    return '';
}
exports.default = clone;
